function [imdec,cput]=deconv2d_vwdblockTI(x,h,lambda,L,sigma,param,iters)
% Block image deconvolution by first block-denoising, then deconvolving with iterative Wiener filter.
% This bears similarities with vaguelet-wavelet decomposition of Abramovitch and Silverman.
%
% 	Input
%		x	Input image.
%		h	PSF image.
%		deg	Degree of Meyer polynomial window 2 <= deg <= 4 (in fact, indexed fro 1 to 3).
%		lambda	Threshold parameter (default, 4.5).
%		L	Block size (default, floor(sqrt(2*J))).
%		sigma	Noise std (default, estimated by MAD).
%		param	Wiener parameter (PSD of noise assumed to be AWGN after denoising).	
%		iters	Number of iterations for the iterative wiener filtering.
%
%	Output
%		imdec	deconvolved image.
%		cput	A vector of CPU time of each step: 
%			+ cpu(1) denoiseing step.
%			+ cpu(2) deconvolution step.
%

if (exist('deg','var')~=1), deg = 1; end;

n = length(x);

% Denoise by block-thresholding in the UDWT domain.
%imden = den2d_blockTI(x,qmf,lambda,L,sigma);
[imden,cputden] = den2d_blockMeyerTI(x,deg,lambda,L,sigma);
cput(1) = sum(cputden);

% Iterative Wiener deconvlution.
t = cputime;
imdec = wienerdeconviter(imden,h,n^2*param,iters);
cput(2) = cputime - t;

