function imdeconv = landweberdeconv(y,psf,alpha,iterations)
% Landweber iterative deconvolution.
% (non-regularized, number of iteratiosn serves as a regularization parameter).
% Degradation equation:
%               y = h*x + eps, 
%		y	Observed image.
%		x	Original image to be recovered whose PSD is Sxx(w).
%		h	PSF (blurring function IR): assumed to be centred at the image center (M/2,N/2) and normalized to a unit mass.
%		*	Circular convolution.
%		eps	Stationary gaussian zero-mean noise with PSD See(w) (i.e. here assumed white gaussian).
%
% We here implement an iterative Landweber restoration expressed in the spectral domain:
%
%             Xest^(t+1)(w) = Xest^(t)(w) + alpha H*(w) (Y(w) - H(w) Xest^(t)(w))
%
%		alpha is typically 1.
%
%

[N,N] = size(y);

if ~exist('psf')
      error('Blur PSF is required.');
end

if length(psf) ~= N
      error('Blur PSF must have the same size as observed image.');
end

if alpha*max(psf(:).^2) >= 2 | alpha*max(psf(:).^2) < 0
      error('Incorrect alpha range: 0 < alpha |H(0)|^2 < 2 ');  % SC for convergence 0 < alpha |H(0)|^2 < 2.
end

FTY = fft2(y);
G = fft2(psf);
LandEst = FTY;
for i = 1:iterations
  residual = FTY - G.*LandEst;
  LandEst = LandEst + alpha * conj(G) .* residual;
  %energy(i) = 0.5*norm(residual/N)^2;
  %plot(energy);drawnow;
end
imdeconv = real(ifft2(LandEst));
