function [imdec,cput]=mrdeconv(x,h,regtype,wavtype,sigma,iters)
% Block image deconvolution by multiresolution support regularization in the TI-wavelet domain.
%
% 	Input
%		x	Input image.
%		h	PSF image.
%		regtye	Type of multiresolution support regularization (default 2):
%				1: Deconvolution by multiresolution Van Cittert's algorithm.
%				2: Deconvolution by multiresolution gradient's algorithm.
%				3: Deconvolution by multiresolution MAP algorithm.
%		wavtype	Type of wavelet (default 1):
%				1: Biorthogonal 7/9 filters, 2: Daubechies filter 4, 3: Haar filter.
%		sigma	Noise std (default, estimated).
%		iters	Number of iterations.
%
%	Output
%		imdec	deconvolved image.
%		cput	cpu time of deconvolution.
%

if (exist('x')~=1), 		error('Provide input image');	end;
if (exist('h')~=1), 		error('Provide input PSF');	end;
if (exist('regtype')~=1), 	regtype = 2; 			end;
if (exist('wavtype')~=1),	wavtype = 1;			end;
if (exist('iters')~=1),		iters   = 100;			end;

fitswrite(rot90(x,-1),'degraded.fits');
fitswrite(rot90(h,-1),'psf.fits');

prog = './mr_deconv';

options = sprintf('-t24 -i%d ',iters);

if (exist('sigma')==1),	options = sprintf('%s -g%f ',options,sigma);	end;

switch regtype
	case 1
	  options = [options '-d1 '];
	case 2
	  options = [options '-d2 '];
	case 3
	  options = [options '-d4 '];
end

switch wavtype
	case 1
	  options = [options '-T1 '];
	case 2
	  options = [options '-T2 '];
	case 3
	  options = [options '-T11 '];
end

options = sprintf('%s degraded.fits psf.fits deconv.fits',options);

%sprintf('%s %s',prog,options)

tic;
unix(sprintf('%s %s',prog,options));
cput = toc;

imdec = fliplr(rot90(fitsread('deconv.fits'),-2));
imdec = shift(imdec,-1);

unix('rm -f degraded.fits psf.fits deconv.fits');
 
