clear all
close all
MC     = 10;
lambda = [3.5 4 4.5 5]; 
L      = [1 2 4 8 16];
deg    = 1;
BSNR   = [10 15 20 25 30 40];
regparam=[10 10 1 1 0.1 0.1];
regprox =[0.5 0.5 0.2 0.2 0.1 0.1];
images = {'barbara_512x512.png','lena512x512.png','nimes512.png','house.png','boat.png','camera.png','peppers256.png'};
randn('state',0);

% PSF
psf = 3;
  
  
for in=1:length(images)
  img = double(imread(images{in}));
  img = 255*img/max(img(:));
  n   = length(img);
  h   = psfgenerate(psf,n);
  % Normalize and FFT
  h = h/sum(h(:));
  H = fft2(h);
  fprintf('\n%s: ',images{in});
  for is=1:length(BSNR)
  for rep=1:MC
	imblur = real(ifft2(fft2(img).*H));
	sigma  = std(imblur(:))*10^(-BSNR(is)/20);
	imobs=imblur+sigma*randn(size(img));
	for il=1:length(lambda)
%     	 for ib=1:length(L)   
% 	   % Block deconvolution.  
% 	   imdec=deconv2d_block(imobs,h,deg,lambda(il),L(ib),sigma);
% 	   isnrblock(in,is,rep,il,ib)=isnr(imobs(:),img(:),imdec(:));
% 	   % Block TI deconvolution.  
% 	   imdecTI=deconv2d_blockTI(imobs,h,deg,lambda(il),L(ib),sigma);
% 	   isnrblockTI(in,is,rep,il,ib)=isnr(imobs(:),img(:),imdecTI(:));
% 	 end
	 % WaveD deconvolution.  
% 	 imdecwaved=deconv2d_term(imobs,h,deg,sqrt(lambda(il)*log(n)),sigma);
% 	 isnrwaved(in,is,rep,il)=isnr(imobs(:),img(:),imdecwaved(:));
	 % TI-WaveD deconvolution.
	 imdecwavedTI=deconv2d_termTI(imobs,h,deg,sqrt(lambda(il)*log(n)/n),sigma);
	 isnrwavedTI(in,is,rep,il)=isnr(imobs(:),img(:),imdecwavedTI(:));
	end
	% TI-VWD block deconvolution.
	imdecvwdTI=deconv2d_vwdblockTI(imobs,h,4.5,4,sigma,regparam(is),20);
	isnrvwdTI(in,is,rep)=isnr(imobs(:),img(:),imdecvwdTI(:));
	% Tikhonov deconvolution with Laplacian regularization operator.
	imdecreg=regdeconv(imobs,h,1);
	isnrreg(in,is,rep)=isnr(imobs(:),img(:),imdecreg(:));
	% Iterative Wiener Filter as proposed in Hillery and Chin 1991.
	imdecwien = wienerdeconviter(imobs,h,n^2*sigma^2,20);
	isnrwien(in,is,rep)=isnr(imobs(:),img(:),imdecwien(:));
	% ForWaRD.
	imdecward = forward(imobs,h);
	isnrward(in,is,rep)=isnr(imobs(:),img(:),imdecward(:));
	% Proximal algorithm deconvolution with the UDWT.
	imdecprox = proximaliteration(imobs, h, regprox(is)*sigma, 1, 'Haar', 1, 5, 0, 100);
	isnrprox(in,is,rep)=isnr(imobs(:),img(:),imdecprox(:));
  end
  progressbar((in-1)*length(BSNR)+is,length(images)*length(BSNR));
  end
end

figure(in);
set(gcf,'Name',sprintf('%s',images{in}),'NumberTitle','off');
subplot(2,5,1)
imagesc(imobs);axis image;rmaxis
title(sprintf('Observed image BPSNR=%g dB',BSNR));
subplot(2,5,2)
imagesc(imdec);axis image;rmaxis
title(sprintf('Block deconvolution ISNR=%g dB',mean(isnrblock)));
subplot(2,5,3)
imagesc(imdecTI);axis image;rmaxis
title(sprintf('Block TI deconvolution ISNR=%g dB',mean(isnrblockTI)));
subplot(2,5,4)
imagesc(imdecwaved);axis image;rmaxis
title(sprintf('WaveD deconvolution ISNR=%g dB',mean(isnrwaved)));
subplot(2,5,5)
imagesc(imdecwavedTI);axis image;rmaxis
title(sprintf('WaveD TI deconvolution ISNR=%g dB',mean(isnrwavedTI)));
subplot(2,5,6)
imagesc(imdecreg);axis image;rmaxis
title(sprintf('Tikhonov deconvolution ISNR=%g dB',mean(isnrreg)));
subplot(2,5,7)
imagesc(imdecwien);axis image;rmaxis
title(sprintf('Iterative Wiener deconvolution ISNR=%g dB',mean(isnrwien)));
subplot(2,5,8)
imagesc(imdecvwdTI);axis image;rmaxis
title(sprintf('TI-VWD-like deconvolution ISNR=%g dB',mean(isnrvwdTI)));
subplot(2,5,9)
imagesc(imdecward);axis image;rmaxis
title(sprintf('ForWaRD deconvolution ISNR=%g dB',mean(isnrward)));
subplot(2,5,10)
imagesc(imdecprox);axis image;rmaxis
title(sprintf('FB proximal algorithm ISNR=%g dB',mean(isnrprox)));
colormap('gray')

save simudecblockdeconv.mat


%
% Part of DBlockToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
%             Jean-Luc Starck, CEA-CNRS
% Created December 2008
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 
