clear all
close all
lambda = 3.5; 
L      = 4;
deg    = 1;
BSNR   = 30;
images = {'barbara_512x512.png','lena512x512.png','nimes512.png','house.png','boat.png','camera.png','peppers256.png'};
randn('state',0);

% PSF
psf = 3;
  
for in=1:length(images)
  img = double(imread(images{in}));
  img = 255*img/max(img(:));
  n   = length(img);
  h   = psfgenerate(psf,n);
  % Normalize and FFT
  h = h/sum(h(:));
  H = fft2(h);
  fprintf('%s: ',images{in});
  imblur = real(ifft2(fft2(img).*H));
  sigma  = std(imblur(:))*10^(-BSNR/20);
  imobs=imblur+sigma*randn(size(img));
  % Block deconvolution.  
  [imdec,cputdec]=deconv2d_block(imobs,h,deg,lambda,L,sigma);
  isnrblock(in)=isnr(imobs(:),img(:),imdec(:));
  % Block TI deconvolution.  
  [imdecTI,cputdecTI]=deconv2d_blockTI(imobs,h,deg,lambda,L,sigma);
  isnrblockTI(in)=isnr(imobs(:),img(:),imdecTI(:));
  % WaveD deconvolution.  
  [imdecwaved,cputwaved]=deconv2d_term(imobs,h,deg,sqrt(lambda*log(n)),sigma);
  isnrwaved(in)=isnr(imobs(:),img(:),imdecwaved(:));
  % TI-WaveD deconvolution.
  [imdecwavedTI,cputwavedTI]=deconv2d_termTI(imobs,h,deg,sqrt(lambda*log(n)/n),sigma);
  isnrwavedTI(in)=isnr(imobs(:),img(:),imdecwavedTI(:));
  % TI-WVD block deconvolution.
  [imdecvwdTI,cputvwdTI]=deconv2d_vwdblockTI(imobs,h,4.5,4,sigma,0.1,20);
  isnrvwdTI(in)=isnr(imobs(:),img(:),imdecvwdTI(:));
  % Tikhonov deconvolution with Laplacian regularization operator.
  tic;imdecreg=regdeconv(imobs,h,1);cputdecreg = toc;
  isnrreg(in)=isnr(imobs(:),img(:),imdecreg(:));
  % Iterative Wiener Filter as proposed in Hillery and Chin 1991.
  tic;imdecwien = wienerdeconviter(imobs,h,n^2*sigma^2,20);cputdecwien = toc;
  isnrwien(in)=isnr(imobs(:),img(:),imdecwien(:));
  % ForWaRD.
  tic;imdecward = forward(imobs,h);cputdecward = toc;
  isnrward(in)=isnr(imobs(:),img(:),imdecward(:));
  % Proximal algorithm deconvolution with the UDWT.
  tic;imdecprox = proximaliteration(imobs, h, 0.1*sigma, 1, 'Haar', 1, 5, 0, 100);cputdecprox = toc;
  isnrprox(in)=isnr(imobs(:),img(:),imdecprox(:));
  figure(in);
  set(gcf,'Name',sprintf('%s',images{in}),'NumberTitle','off');
  subplot(2,5,1)
  imagesc(imobs);axis image;rmaxis
  title(sprintf('Observed image BPSNR=%g dB',BSNR));
  subplot(2,5,2)
  imagesc(imdec);axis image;rmaxis
  title(sprintf('Block deconvolution ISNR=%g dB',isnrblock(in)));
  subplot(2,5,3)
  imagesc(imdecTI);axis image;rmaxis
  title(sprintf('Block TI deconvolution ISNR=%g dB',isnrblockTI(in)));
  subplot(2,5,4)
  imagesc(imdecwaved);axis image;rmaxis
  title(sprintf('WaveD deconvolution ISNR=%g dB',isnrwaved(in)));
  subplot(2,5,5)
  imagesc(imdecwavedTI);axis image;rmaxis
  title(sprintf('WaveD TI deconvolution ISNR=%g dB',isnrwavedTI(in)));
  subplot(2,5,6)
  imagesc(imdecreg);axis image;rmaxis
  title(sprintf('Tikhonov deconvolution ISNR=%g dB',isnrreg(in)));
  subplot(2,5,7)
  imagesc(imdecwien);axis image;rmaxis
  title(sprintf('Iterative Wiener deconvolution ISNR=%g dB',isnrwien(in)));
  subplot(2,5,8)
  imagesc(imdecvwdTI);axis image;rmaxis
  title(sprintf('TI-VWD-like deconvolution ISNR=%g dB',isnrvwdTI(in)));
  subplot(2,5,9)
  imagesc(imdecward);axis image;rmaxis
  title(sprintf('ForWaRD deconvolution ISNR=%g dB',isnrward(in)));
  subplot(2,5,10)
  imagesc(imdecprox);axis image;rmaxis
  title(sprintf('FB proximal algorithm ISNR=%g dB',isnrprox(in)));
  colormap('gray')
  drawnow
  saveas(gcf,sprintf('%s_SNR=%d.fig',images{in},BSNR),'fig');
  saveas(gcf,sprintf('%s_SNR=%d.eps',images{in},BSNR),'epsc');
  
  str = whos('cput*');
  for is=1:length(str)
   fprintf('sum(%s) = %f\n',str(is).name,eval(sprintf('sum(%s)',str(is).name)))
  end
end

tilefigs


%
% Part of DBlockToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
%             Jean-Luc Starck, CEA-CNRS
% Created December 2008
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 
